﻿// Created by: X
// Web: http://www.createdbyx.com/
// Date: January 28, 2008
// =============================================
using System.Collections;
//using System.Data;
using System.Collections.Generic;
using System;


namespace MMO_Objects
{
    public class GenericCollection<T> : IList<T>, IDisposable
    {

        public delegate void BeforeAddedEventHandler(object sender, ref bool Cancel, T Value);
        public event BeforeAddedEventHandler BeforeAdded;

        public delegate void AfterAddedEventHandler(object sender, T Value);
        public event AfterAddedEventHandler AfterAdded;

        public delegate void BeforeInsertEventHandler(object sender, ref bool Cancel, int Index, T Value);
        public event BeforeInsertEventHandler BeforeInsert;

        public delegate void AfterInsertEventHandler(object sender, int Index, T Value);
        public event AfterInsertEventHandler AfterInsert;

        public delegate void BeforeRemoveEventHandler(object sender, ref bool Cancel, T Value);
        public event BeforeRemoveEventHandler BeforeRemove;

        public delegate void AfterRemoveEventHandler(object sender, T Value);
        public event AfterRemoveEventHandler AfterRemove;

        public delegate void BeforeRemoveAtEventHandler(object sender, ref bool Cancel, int Index);
        public event BeforeRemoveAtEventHandler BeforeRemoveAt;

        public delegate void AfterRemoveAtEventHandler(object sender, int Index);
        public event AfterRemoveAtEventHandler AfterRemoveAt;

        public delegate void BeforeChangeEventHandler(object sender, ref bool Cancel, int Index, T Value);
        public event BeforeChangeEventHandler BeforeChange;

        public delegate void AfterChangeEventHandler(object sender, int Index, T Value);
        public event AfterChangeEventHandler AfterChange;

        public delegate void BeforeClearEventHandler(object sender, ref bool Cancel);
        public event BeforeClearEventHandler BeforeClear;

        public delegate void AfterClearEventHandler(object sender);
        public event AfterClearEventHandler AfterClear;


        protected internal System.Collections.Generic.List<T> mobjItems = new System.Collections.Generic.List<T>();
        protected internal bool mblnCallDisposeOnItems = true;


        public virtual bool CallDisposeOnItems
        {
            get
            {
                return mblnCallDisposeOnItems;
            }
            set
            {
                mblnCallDisposeOnItems = value;
            }
        }

        public virtual void Add(T item)
        {
            if (item == null) return;
            bool Cancel = false;
            if (BeforeAdded != null) BeforeAdded(this, ref Cancel, item);

            // check if canceled
            if (Cancel) return;

            // add layer
            mobjItems.Add(item);
            // raise after event
            if (AfterAdded != null) AfterAdded(this, item);
        }

        public virtual void Add(T[] item)
        {
            if (item == null) return;
            int idx = 0;
            while (idx < item.Length)
            {
                this.Add(item[idx]);
                idx++;
            }
        }

        public virtual int Count
        {
            get { return mobjItems.Count; }
        }

        public bool IsReadOnly
        {
            get { return false; }
        }

        //public virtual new System.Collections.Generic.IEnumerator<T> IEnumerable()
        //{
        //    return mobjItems.GetEnumerator;
        //}

        IEnumerator<T> IEnumerable<T>.GetEnumerator()
        {
            return mobjItems.GetEnumerator();
        }

        public virtual IEnumerator GetEnumerator()
        {
            return mobjItems.GetEnumerator();
        }


        public virtual void Clear()
        {
            bool cancel = false;
            if (BeforeClear != null) BeforeClear(this, ref cancel);

            if (cancel) return;

            // clear items
            if (mblnCallDisposeOnItems)
            {
                foreach (object i in mobjItems)
                {
                    if (i is IDisposable)
                    {
                        ((IDisposable)i).Dispose();
                    }
                }
            }
            mobjItems.Clear();

            if (AfterClear != null) AfterClear(this);
        }

        public virtual bool Remove(T item)
        {
            if (!mobjItems.Contains(item)) return true;

            bool Cancel = false;
            if (BeforeRemove != null) BeforeRemove(this, ref Cancel, item);

            // check if canceled
            if (Cancel) return false;

            // add layer
            mobjItems.Remove(item);
            // raise after event
            if (AfterRemove != null) AfterRemove(this, item);

            return true;
        }

        public virtual int IndexOf(T item)
        {
            return mobjItems.IndexOf(item);
        }

        public virtual void Insert(int index, T item)
        {
            bool Cancel = false;
            if (BeforeInsert != null) BeforeInsert(this, ref Cancel, index, item);

            // check if canceled
            if (Cancel) return;

            // add layer
            mobjItems.Insert(index, item);
            // raise after event
            if (AfterInsert != null) AfterInsert(this, index, item);
        }

        public virtual T this[int index]
        {
            get
            {
                return mobjItems[index];
            }
            set
            {
                bool Cancel = false;
                if (BeforeChange != null) BeforeChange(this, ref Cancel, index, value);

                // check if canceled
                if (Cancel) return;

                // add layer
                mobjItems[index] = value;
                // raise after event
                if (AfterChange != null) AfterChange(this, index, value);
            }
        }

        public virtual void RemoveAt(int index)
        {
            if (index < 0 || index > mobjItems.Count - 1) return;

            bool Cancel = false;
            if (BeforeRemoveAt != null) BeforeRemoveAt(this, ref Cancel, index);

            // check if canceled
            if (Cancel) return;

            // add layer
            mobjItems.RemoveAt(index);
            // raise after event
            if (AfterRemoveAt != null) AfterRemoveAt(this, index);
        }

        public virtual void CopyTo(T[] items, int arrayindex)
        {
            this.mobjItems.CopyTo(items, arrayindex);
        }

        public virtual bool Contains(T item)
        {
            return mobjItems.Contains(item);
        }


        #region " IDisposable Support "
        private bool disposedValue = false; // To detect redundant calls

        // IDisposable
        protected virtual void Dispose(bool disposing)
        {
            if (!this.disposedValue)
            {
                if (disposing)
                {
                    // TODO: free unmanaged resources when explicitly called
                    if (mblnCallDisposeOnItems)
                    {
                        foreach (object i in mobjItems)
                        {
                            if (i is IDisposable)
                            {
                                ((IDisposable)i).Dispose();
                            }
                        }
                    }
                    mobjItems.Clear();
                    mobjItems = null;
                }

                // TODO: free shared unmanaged resources
            }
            this.disposedValue = true;
        }

        // This code added by Visual Basic to correctly implement the disposable pattern.
        public void Dispose()
        {
            // Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
            Dispose(true);
            GC.SuppressFinalize(this);
        }
        #endregion


    }

}
